package AlgLin;

import java.io.*;
import java.util.*;

public class Vecteur extends Matrice {

    // Constructeur avec un paramètre entier
    Vecteur(int nbligne) {
        super(nbligne, 1);
    }


    // Constructeur avec un tableau de coefficients
    public Vecteur(double[] coefficients) {
        super(coefficients.length, 1);
        for (int i = 0; i < coefficients.length; i++) {
            this.coefficient[i][0] = coefficients[i];
        }
    }

    // Constructeur avec un fichier
    public Vecteur(String fichier) {
        super(0,0);
        try {
            Scanner sc = new Scanner(new File(fichier));
            int taille = sc.nextInt();
            this.coefficient = new double[taille][1];
            for (int i = 0; i < taille; i++) {
                this.coefficient[i][0] = sc.nextDouble();
            }
            sc.close();
        } catch (FileNotFoundException e) {
        	System.out.println("Fichier absent");
        } catch (Exception e) {
            System.out.println("Erreur lors de la lecture du fichier : ");
        }
    }

    // Méthode qui renvoie la taille du vecteur
    public int taille() {
        return this.coefficient.length;
    }
    
    
 // Méthode qui renvoie le coefficient à une position donnée
    public double getCoefficient(int position) {
  
        return this.coefficient[position][0];
    }

    // Méthode pour modifier un coefficient à une position donnée
    public void setCoefficient(int position, double valeur) {
     
        this.coefficient[position][0] = valeur;
    }

 

 // Méthode statique pour calculer le produit scalaire de deux vecteurs sans vérification préalable
    public static double produitScalaire(Vecteur v1, Vecteur v2) {
        double produit = 0;
        for (int i = 0; i < v1.taille(); i++) {
            produit += v1.getCoefficient(i) * v2.getCoefficient(i);
        }
        return produit;
    }

    // Méthode statique  pour calculer le produit scalaire de deux vecteurs avec vérification préalable
    public static double verifProduitScalaire(Vecteur v1, Vecteur v2) throws Exception {
        if (v1.taille() != v2.taille()) {
            throw new Exception("Les vecteurs n'ont pas la même taille pour un produit scalaire.");
        }
        
        double produit = 0;
        for (int i = 0; i < v1.taille(); i++) {
            produit += v1.getCoefficient(i) * v2.getCoefficient(i);
        }

        return produit;
    }
    
    
    // Méthode toString pour afficher le vecteur
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder("Vecteur : [");
        for (int i = 0; i < this.taille(); i++) {
            sb.append(this.coefficient[i][0]);
            if (i < this.taille() - 1) sb.append(", ");
        }
        sb.append("]");
        return sb.toString();
    }

    public static void main(String[] args) throws Exception {

    	double[] tableau1 = {1.0, 2.0, 3.0};
        Vecteur v1 = new Vecteur(tableau1);
        System.out.println("Construction d'un vecteur par affectation d'un tableau : \n");
        System.out.println(v1);
        System.out.println(); 

        Vecteur v2 = new Vecteur("src/AlgLin/vecteur1.txt");
        System.out.println("Construction d'un vecteur par lecture d'un fichier :");
        System.out.println(v2);
        System.out.println(); 

        Vecteur v3 = new Vecteur(5);
        System.out.println("Construction d'un vecteur de taille 5 :");
        System.out.println(v3);
        System.out.println();

        System.out.println("Taille du vecteur v1 : " + v1.taille());
        System.out.println(); 

        System.out.println("Coefficient à la position 1 dans le vecteur v1 : " + v1.getCoefficient(1));
        System.out.println(); 

        System.out.println("Modification du coefficient à la position 1 de v1 à la valeur 10.0");
        v1.setCoefficient(1, 10.0);
        System.out.println("Vecteur v1 après modification :");
        System.out.println(v1);
        System.out.println(); 

       
        Vecteur v4 = new Vecteur(new double[]{4.0, 5.0, 6.0});
        double produit = Vecteur.produitScalaire(v1, v4);
        System.out.println("Produit scalaire de v1 et v4 :");
        System.out.println(produit);
        System.out.println(); 

        double produitVerifie = Vecteur.verifProduitScalaire(v1, v4);
        System.out.println("Produit scalaire de v1 et v4 (avec vérification) :");
        System.out.println(produitVerifie);
        
    }

}
